<?php
/*******************************************************************
 *			
 * 			Fichier			:	class.attach.php 
 * 			Cr le			:	05 avril 2003 
 * 			Dernire modif	:	13 juin 2003 
 * 			Email			:	wascripts@phpcodeur.net 
 * 
 * 				Copyright  2002-2003 phpCodeur
 * 
 *******************************************************************/

/*******************************************************************
 *  This program is free software; you can redistribute it and/or 
 * 	modify it under the terms of the GNU General Public License as 
 * 	published by the Free Software Foundation; either version 2 of 
 *  the License, or (at your option) any later version. 
 *******************************************************************/


if( !defined('CLASS_ATTACH_INC') )
{

define('CLASS_ATTACH_INC', 1); 

/*
 * Class Attach
 * 
 * Gestion des fichiers joints des newsletters
 */ 
class Attach {
	
	var $upload_path 	= ''; 
	var $use_ftp 		= false; 
	var $ftp_path 		= ''; 
	var $connect_id		= ''; 
	
	/*
	 * Attach::Attach()
	 * 
	 * Intialisation des variables de la classe
	 * Intialisation de la connexion au serveur ftp le cas chant
	 * 
	 * @return void
	 */
	function Attach()
	{
		global $nl_config, $waroot; 
		
		$this->upload_path	= $waroot . $nl_config['upload_path']; 
		$this->use_ftp		= $nl_config['use_ftp']; 
		
		if( $this->use_ftp )
		{
			$resultat = $this->connect_to_ftp(
				$nl_config['ftp_server'],
				$nl_config['ftp_user'],
				$nl_config['ftp_pass'],
				$nl_config['ftp_pasv'],
				$nl_config['ftp_path']
			); 
			
			if( $resultat['error'] )
			{
				trigger_error($resultat['message'], ERROR); 
			}
			
			$this->connect_id = $resultat['connect_id']; 
			$this->ftp_path	  = $nl_config['ftp_path']; 
		}
	}
	
	/*
	 * Attach::connect_to_ftp()
	 * 
	 * Fonction de connexion au serveur ftp
	 * La fonction a t affranchi de faon  tre utilisable sans crer 
	 * une instance de la classe. (pour tester la connexion dans la config. gnrale)
	 * 
	 * @param string  $ftp_server	: Nom du serveur ftp
	 * @param string  $ftp_user		: Nom d'utilisateur si besoin
	 * @param string  $ftp_pass		: Mot de passe si besoin
	 * @param integer $ftp_pasv		: Mode actif ou passif
	 * @param string  $ftp_path		: Chemin vers le dossier des fichiers joints
	 * 
	 * @return array
	 */
	function connect_to_ftp($ftp_server, $ftp_user, $ftp_pass, $ftp_pasv, $ftp_path)
	{
		if( !($connect_id = @ftp_connect($ftp_server)) )
		{
			return array('error' => true, 'message' => 'Ftp_unable_connect'); 
		}
		
		if( $ftp_user != '' && $ftp_pass != '' )
		{
			if( !@ftp_login($connect_id, $ftp_user, $ftp_pass) )
			{
				return array('error' => true, 'message' => 'Ftp_error_login'); 
			}
		}
		
		if( !@ftp_pasv($connect_id, $ftp_pasv) )
		{
			return array('error' => true, 'message' => 'Ftp_error_mode'); 
		}
		
		if( !@ftp_chdir($connect_id, $ftp_path) )
		{
			return array('error' => true, 'message' => 'Ftp_error_path'); 
		}
		
		return array('error' => false, 'connect_id' => $connect_id); 
	}
		
	/*
	 * Attach::joined_file_exists()
	 * 
	 * Verifie la prsence du fichier demand dans le dossier 
	 * des fichier joints ou sur le ftp
	 * 
	 * @param string  $filename		: Nom du fichier
	 * @param boolean $error		: True si une erreur s'est produite
	 * @param array   $msg_error	: Tableau des erreurs
	 * 
	 * @return integer
	 */
	function joined_file_exists($filename, &$error, &$msg_error)
	{
		global $lang; 
		
		$file_exists = false; 
		$filesize	 = 0; 
		
		if( $this->use_ftp )
		{
			$listing = @ftp_rawlist($this->connect_id, $this->ftp_path); 
			
			if( is_array($listing) && count($listing) )
			{
				foreach( $listing AS $line_info )
				{
					if( preg_match('/^\s*([d-])[rwxst-]{9} .+ ([0-9]*) [a-zA-Z]+ [0-9:\s]+ (.+)$/i', $line_info, $matches) )
					{
						if( $matches[1] != 'd' && $matches[3] == $filename )
						{
							$file_exists = true; 
							$filesize	 = $matche[2]; 
							
							break;
						}
					}
				}
			}
		}
		else if( file_exists(wa_realpath($this->upload_path . $filename)) )
		{
			$file_exists = true; 
			$filesize 	 = filesize(wa_realpath($this->upload_path . $filename)); 
		}
		
		if( !$file_exists )
		{
			$error = TRUE; 
			$msg_error[] = sprintf($lang['Message']['File_not_exists'], htmlspecialchars($filename)); 
		}
		
		return $filesize; 
	}
		
	/*
	 * Attach::make_filename()
	 * 
	 * Gnration d'un nom de fichier unique
	 * Fonction rcursive
	 * 
	 * @param string $prev_physical_name	: Nom du fichier temporaire prcdemment gnr et refus
	 * 
	 * @return string
	 */
	function make_filename($prev_physical_name = '')
	{
		global $db; 
		
		$physical_name = md5( uniqid( rand() ) ) . '.dl'; 
		
		if( $physical_name != $prev_physical_name )
		{
			$sql = "SELECT COUNT(file_id) AS test_name 
					FROM " . JOINED_FILES_TABLE . " 
					WHERE file_physical_name = '$physical_name'"; 
			if( !($result = $db->query($sql)) )
			{
				trigger_error('Impossible de tester la table des fichiers joints', ERROR); 
			}
			
			$test_name = $db->result($result, 0, 'test_name'); 
		}
		else
		{
			$test_name = true; 
		}
		
		return ( $test_name ) ? $this->make_filename($physical_name) : $physical_name; 
	}
	
	/*
	 * Attach::file_on_server()
	 * 
	 * Fonction se chargeant de verifier la prsence du fichier spcifi.
	 * Effectue les vrifications ncessaires et ajoute une entre dans les tables de 
	 * gestion des fichiers joints
	 * 
	 * @param mixed   $file			: Id d'un fichier dja joint  utiliser/
	 * 								  Nom du fichier upload manuellement
	 * @param integer $log_id		: Identifiant du log
	 * @param boolean $error		: True si une erreur survient
	 * @param array   $msg_error	: Tableau des messages d'erreur
	 * 
	 * @return void
	 */
	function file_on_server($file, $log_id, &$error, &$msg_error)
	{
		global $waroot, $db, $lang; 
		
		//
		// Si $file est un entier, on rcupre les donnes du fichier joint  rutiliser
		// Sinon, c'est un fichier upload manuellement
		//
		if( is_numeric($file) )
		{
			$file_id = $file; 
			
			$sql = "SELECT file_real_name, file_physical_name, file_mimetype 
					FROM " . JOINED_FILES_TABLE . " 
					WHERE file_id = " . $file_id; 
			if( !($result = $db->query($sql)) )
			{
				trigger_error('Impossible de rcuprer les donnes sur ce fichier', ERROR); 
			}
			
			if( $row = $db->fetch_array($result) )
			{
				$data = $row; 
			}
			else
			{
				$error = TRUE; 
				$msg_error[] = sprintf($lang['Message']['File_not_exists'], ''); 
			}
		}
		else
		{
			$file_id = 0; 
			
			$data = array(); 
			$data['file_real_name'] 	= $file; 
			$data['file_physical_name'] = $file; 
		}
		
		if( !$error )
		{
			$file_ext = substr($data['file_real_name'], (strrpos($data['file_real_name'], '.') + 1)); 
			
			if( $file_ext == '' )
			{
				$file_ext = 'wa'; 
			}
			
			if( !$file_id )
			{
				include($waroot . 'includes/class.mailer.php'); 
				$data['file_mimetype'] = Mailer::mime_type($file_ext); 
			}
			
			//
			// On verifie si le fichier est bien prsent sur le serveur
			//
			$data['file_size'] = $this->joined_file_exists($data['file_physical_name'], $error, $msg_error); 
		}
		
		if( !$error && !$this->check_filename($data['file_real_name']) )
		{
			$error = TRUE; 
			$msg_error[] = $lang['Message']['Invalid_filename']; 
		}
		
		if( !$error && !$this->check_extension($file_ext) )
		{
			$error = TRUE; 
			$msg_error[] = $lang['Message']['Invalid_ext']; 
		}
		
		$total_size = 0; 
		if( !$error && !$this->check_maxsize($log_id, $data['file_size'], $total_size) )
		{
			$error = TRUE; 
			$msg_error[] = sprintf($lang['Message']['weight_too_big'], ($nl_config['max_filesize'] - $total_size)); 
		}
		
		if( !$error )
		{
			if( $file_id )
			{
				$this->insert_data($log_id, $file_id); 
			}
			else
			{
				$this->insert_data($log_id, $data); 
			}
		}
		
		$this->quit(); 
	}
		
	/*
	 * Attach::upload_file()
	 * 
	 * Fonction se chargeant de verifier le fichier upload 
	 * Effectue les vrifications ncessaires et ajoute une entre dans les tables de 
	 * gestion des fichiers joints
	 * 
	 * @param array   $filedata		: Tableau de donnes du fichier upload via le formulaire
	 * @param integer $log_id		: Identifiant du log
	 * @param boolean $error		: True si une erreur survient
	 * @param array   $msg_error	: Tableau des messages d'erreur
	 * 
	 * @return void
	 */
	function upload_file($filedata, $log_id, &$error, &$msg_error)
	{
		global $lang; 
		
		$real_filename	= $filedata['name']; 
		$filesize		= $filedata['size']; 
		$mime_type		= $filedata['type']; 
		$tmp_filename	= $filedata['tmp_name']; 
		$errno_code		= ( isset($filedata['error']) ) ? intval($filedata['error']) : UPLOAD_ERR_OK; 
		
		$file_ext = substr($real_filename, (strrpos($real_filename, '.') + 1)); 
		
		if( $file_ext == '' )
		{
			$file_ext = 'wa'; 
		}
		
		//
		// Si l'upload a chou, on rcupre le texte de l'erreur survenue
		//
		if( !$error && !is_uploaded_file($tmp_filename) )
		{
			$error = TRUE; 
			
			switch( $errno_code )
			{
				case UPLOAD_ERR_INI_SIZE:
					$msg_error[] = $lang['Message']['Upload_error_1']; 
					break;
				
				case UPLOAD_ERR_FORM_SIZE:
					$msg_error[] = $lang['Message']['Upload_error_2']; 
					break;
				
				case UPLOAD_ERR_PARTIAL:
					$msg_error[] = $lang['Message']['Upload_error_3']; 
					break;
				
				case UPLOAD_ERR_NO_FILE:
					$msg_error[] = $lang['Message']['Upload_error_4']; 
					break;
				
				default:
					$msg_error[] = $lang['Message']['Upload_error_5']; 
					break;
			}
		}
		
		if( !$error && !$this->check_filename($real_filename) )
		{
			$error = TRUE; 
			$msg_error[] = $lang['Message']['Invalid_filename']; 
		}
		
		if( !$error && !$this->check_extension($file_ext) )
		{
			$error = TRUE; 
			$msg_error[] = $lang['Message']['Invalid_ext']; 
		}
		
		$total_size = 0; 
		if( !$error && !$this->check_maxsize($log_id, $filesize, $total_size) )
		{
			$error = TRUE; 
			$msg_error[] = sprintf($lang['Message']['weight_too_big'], ($nl_config['max_filesize'] - $total_size)); 
		}
		
		//
		// On le dplace le fichier upload dans le dossier des fichiers joints ou sur le serveur ftp
		//
		if( !$error )
		{
			$physical_filename = $this->make_filename(); 
			
			if( $this->use_ftp )
			{
				$mode = $this->get_mode($mime_type); 
				
				if( !@ftp_put($this->connect_id, $physical_filename, $tmp_filename, $mode) )
				{
					$error = TRUE; 
					$msg_error[] = $lang['Message']['Ftp_error_put']; 
				}
				else
				{
					@ftp_site($this->connect_id, 'CHMOD 0644 ' . $physical_filename); 
				}
			}
			else
			{
				if( !@move_uploaded_file($tmp_filename, $this->upload_path . $physical_filename) )
				{
					$error = TRUE; 
					$msg_error[] = $lang['Message']['Upload_error_5']; 
				}
				else
				{
					@chmod($this->upload_path . $physical_filename, 0644); 
				}
			}
		}
		
		if( !$error )
		{
			$data = array(); 
			$data['file_real_name']		= $real_filename; 
			$data['file_physical_name']	= $physical_filename; 
			$data['file_size']			= $filesize; 
			$data['file_mimetype']		= $mime_type; 
			
			$this->insert_data($log_id, $data); 
		}
		
		$this->quit(); 
	}
	
	
	/*
	 * Attach::check_filename()
	 * 
	 * Vrification de la validit du nom de fichier
	 * 
	 * @param string $filename	: Nom du fichier
	 * 
	 * @return boolean
	 */
	function check_filename($filename)
	{
		return preg_match('/[^\\:*\/?<">|]/', $filename); 
	}
	
	/*
	 * Attach::check_extension()
	 * 
	 * Vrification de la validit du nom de fichier
	 * 
	 * @param string $ext	: Extension du fichier
	 * 
	 * @return integer
	 */
	function check_extension($ext)
	{
		global $db, $admindata; 
		
		$sql = "SELECT COUNT(fe_id) AS test_extension 
				FROM " . FORBIDDEN_EXT_TABLE . " 
				WHERE fe_ext = '$ext' AND liste_id = " . $admindata['session_liste']; 
		if( !($result = $db->query($sql)) )
		{
			trigger_error('Impossible de tester la table des extensions interdites',  ERROR); 
		}
		
		return ( $db->result($result, 0, 'test_extension') ) ? false : true; 
	}
	
	/*
	 * Attach::check_extension()
	 * 
	 * On vrifie si la taille du fichier fait dpasser la taille du log par rapport  la valeur max
	 * 
	 * @param integer $log_id		: Identifiant du log
	 * @param integer $filesize		: Taille du fichier
	 * @param integer $total_size	: Taille totale du log
	 * 
	 * @return boolean
	 */
	function check_maxsize($log_id, $filesize, &$total_size)
	{
		global $db, $nl_config; 
		
		$sql = "SELECT SUM(jf.file_size) AS total_size 
				FROM " . JOINED_FILES_TABLE . " AS jf, " . LOG_FILES_TABLE . " AS lf 
				WHERE jf.file_id = lf.file_id AND lf.log_id = " . $log_id; 
		if( !($result = $db->query($sql)) )
		{
			trigger_error('Impossible d\'obtenir la somme du poids des fichiers joints', ERROR); 
		}
		
		$total_size = $db->result($result, 0, 'total_size'); 
		
		return ( ($total_size + $filesize) > $nl_config['max_filesize'] ) ? false : true; 
	}
	
	/*
	 * Attach::insert_data()
	 * 
	 * Insertion des donnes dans la base de donnes
	 * 
	 * @param integer $log_id		: Identifiant du log
	 * @param mixed   $filedata		: Identifiant du fichier rutilis/donnes du fichier upload 
	 * 								  manuellement ou via le formulaire
	 * 
	 * @return void
	 */
	function insert_data($log_id, $filedata)
	{
		global $db; 
		
		if( is_array($filedata) )
		{
			if( !$db->query_build('INSERT', JOINED_FILES_TABLE, $filedata) )
			{
				trigger_error('Impossible d\'insrer les donnes du fichier dans la base de donnes', ERROR); 
			}
			
			$file_id = $db->next_id(); 
		}
		else
		{
			$file_id = $filedata; 
		}
		
		$sql = "INSERT INTO " . LOG_FILES_TABLE . " (log_id, file_id) 
				VALUES($log_id, $file_id)"; 
		if( !$db->query($sql) )
		{
			trigger_error('Impossible d\'insrer la jointure dans la table log_files', ERROR); 
		}
	}
	
	/*
	 * Attach::download_file()
	 * 
	 * Tlchargement d'un fichier joint
	 * 
	 * @param integer $file_id	: Identifiant du fichier joint
	 * 
	 * @return void
	 */
	function download_file($file_id)
	{
		global $waroot, $db, $admindata, $lang; 
		
		$sql = "SELECT f.file_real_name, f.file_physical_name, f.file_size, f.file_mimetype 
				FROM " . JOINED_FILES_TABLE . " AS f, " . LOG_FILES_TABLE . " AS lf, " . LOG_TABLE . " AS l 
				WHERE f.file_id = $file_id AND lf.file_id = f.file_id AND lf.log_id = l.log_id 
				  AND l.liste_id = " . $admindata['session_liste']; 
		if( !($result = $db->query($sql)) )
		{
			trigger_error('Impossible d\'obtenir les donnes sur ce fichier', ERROR); 
		}
		
		if( $row = $db->fetch_array($result) )
		{
			if( $this->use_ftp )
			{
				$mode = $this->get_mode($row['file_mimetype']); 
				
				$tmp_path = ( @ini_get('open_basedir') ) ? wa_tmp_path : '/tmp'; 
				$tmp_filename = tempnam($tmp_path, uniqid(rand()) . 'wa0'); 
				
				if( !@ftp_get($this->connect_id, $tmp_filename, $row['file_physical_name'], $mode) )
				{
					trigger_error('Ftp_error_get', ERROR); 
				}
			}
			else
			{
				$tmp_filename = wa_realpath($this->upload_path . $row['file_physical_name']); 
			}
			
			$filedata = ''; 
			$fp = @fopen($tmp_filename, 'rb'); 
			while( !@feof($fp) )
			{
				$filedata .= fread($fp, 1024); 
			}
			fclose($fp); 
			
			$this->quit(); 
			$this->send_file($row['file_real_name'], $row['file_mimetype'], $filedata, $row['file_size']); 
		}
		
		trigger_error(sprintf($lang['Message']['File_not_exists'], ''), MESSAGE); 
	}
	
	/*
	 * Attach::ftp_to_tmp()
	 * 
	 * Dplacement du fichier demand du serveur ftp vers le dossier temporaire du script
	 * Retourne le nom du fichier temporaire cr
	 * 
	 * @param array $data	: Donnes du fichier joint
	 * 
	 * @return string
	 */
	function ftp_to_tmp($data)
	{
		$mode = $this->get_mode($data['file_mimetype']); 
		
		$tmp_path = ( @ini_get('open_basedir') ) ? wa_tmp_path : '/tmp'; 
		$tmp_filename = tempnam($tmp_path, uniqid(rand()) . 'wa1'); 
		
		if( !@ftp_get($this->connect_id, $tmp_filename, $data['file_physical_name'], $mode) )
		{
			trigger_error('Ftp_error_get', ERROR); 
		}
		
		return $tmp_filename; 
	}
	
	/*
	 * Attach::get_mode()
	 * 
	 * Mode  utiliser ascii ou binaire
	 * 
	 * @param string $mimetype	: Mime type du fichier concern
	 * 
	 * @return integer
	 */
	function get_mode($mimetype)
	{
		$mode = FTP_BINARY; 
		if( preg_match('/text|html/i', $mimetype) )
		{
			$mode = FTP_ASCII; 
		}
		
		return $mode; 
	}
	
	/*
	 * Attach::delete_joined_files()
	 * 
	 * Fonction de suppression de fichiers joints
	 * Retourne le nombre des fichiers supprims, en cas de succs
	 * 
	 * @param boolean $massive_delete	: Si true, suppression des fichiers joints du ou des logs concerns
	 * @param mixed   $log_id_ary		: id ou tableau des id des logs concerns
	 * @param mixed   $file_id_ary		: id ou tableau des id des fichiers joints concerns (si $massive_delete  false)
	 * 
	 * @return mixed
	 */
	function delete_joined_files($massive_delete, $log_id_ary, $file_id_ary = array())
	{
		global $db; 
		
		if( !is_array($log_id_ary) )
		{
			$log_id_ary = array($log_id_ary); 
		}
		
		if( !is_array($file_id_ary) )
		{
			$file_id_ary = array($file_id_ary); 
		}
		
		if( count($log_id_ary) )
		{
			if( $massive_delete )
			{
				$sql = "SELECT file_id 
						FROM " . LOG_FILES_TABLE . " 
						WHERE log_id IN(" . implode(', ', $log_id_ary) . ") 
						GROUP BY file_id"; 
				if( !($result = $db->query($sql)) )
				{
					trigger_error('Impossible d\'obtenir la liste des fichiers', ERROR); 
				}
				
				$file_id_ary = array(); 
				while( $row = $db->fetch_array($result) )
				{
					$file_id_ary[] = $row['file_id']; 
				}
			}
			
			if( count($file_id_ary) )
			{
				$filename_ary = array(); 
				
				$sql = "SELECT lf.file_id, jf.file_physical_name 
						FROM " . LOG_FILES_TABLE . " AS lf, " . JOINED_FILES_TABLE . " AS jf 
						WHERE lf.file_id IN(" . implode(', ', $file_id_ary) . ") 
						  AND lf.file_id = jf.file_id 
						GROUP BY lf.file_id 
						HAVING COUNT(lf.file_id) = 1"; 
				if( !($result = $db->query($sql)) )
				{
					trigger_error('Impossible d\'obtenir la liste des fichiers  supprimer', ERROR); 
				}
				
				if( $row = $db->fetch_array($result) )
				{
					$id_ary = array(); 
					
					do
					{
						$id_ary[]  		= $row['file_id']; 
						$filename_ary[] = $row['file_physical_name']; 
					}
					while( $row = $db->fetch_array($result) ); 
					
					$sql = "DELETE FROM " . JOINED_FILES_TABLE . " 
							WHERE file_id IN(" . implode(', ', $id_ary) . ")"; 
					if( !$db->query($sql) )
					{
						trigger_error('Impossible de supprimer les entres inutiles de la table des fichiers joints', ERROR); 
					}
				}
				
				$sql = "DELETE FROM " . LOG_FILES_TABLE . " 
					WHERE log_id IN(" . implode(', ', $log_id_ary) . ") 
					  AND file_id IN(" . implode(', ', $file_id_ary) . ")"; 
				if( !$db->query($sql) )
				{
					trigger_error('Impossible de supprimer les entres de la table log_files', ERROR); 
				}
				
				//
				// Suppression physique des fichiers joints devenus inutiles
				//
				foreach( $filename_ary AS $filename )
				{
					if( $this->use_ftp )
					{
						if( !@ftp_delete($this->connect_id, $filename) )
						{
							trigger_error('Ftp_error_del', ERROR); 
						}
					}
					else
					{
						$this->remove_file(wa_realpath($this->upload_path . $filename)); 
					}
				}
				
				return count($filename_ary); 
			}// end count file_id_ary
		}// end count log_id_ary
		
		return false; 
	}
	
	/*
	 * Attach::remove_file()
	 * 
	 * Suppression d'un fichier du serveur
	 * 
	 * @param string $filename	: Nom du fichier sur le serveur
	 * 
	 * @return void
	 */
	function remove_file($filename)
	{
		@chmod($filename, 0777); 
		@unlink($filename); 
		
		if( file_exists($filename) )
		{
			@system('del ' . str_replace('/', '\\', $filename)); 
		}
	}
	
	/*
	 * Attach::send_file()
	 * 
	 * Fonction d'envois des enttes ncessaires au tlchargements, puis 
	 * des donnes du fichier  tlcharger
	 * 
	 * @param string  $filename		: Nom rel du fichier
	 * @param string  $mime_type	: Mime type du fichier
	 * @param string  $filedata		: Contenu du fichier
	 * @param integer $filesize		: Taille du fichier
	 * 
	 * @return void
	 */
	function send_file($filename, $mime_type, $filedata, $filesize = 0)
	{
        if( $mime_type == '' || $mime_type == 'application/octetstream' || $mime_type == 'application/octet-stream' )
        {
			if( WA_USER_BROWSER == 'msie' || WA_USER_BROWSER == 'opera' )
			{
			$mime_type = 'application/octetstream'; 
			}
			else
			{
				$mime_type = 'application/octet-stream'; 
			}
		}
		
		header('Expires: 0'); 
		header('Content-Transfer-Encoding: none'); 
		header('Content-Type: ' . $mime_type); 
		
		if( WA_USER_BROWSER == 'msie' || WA_USER_BROWSER == 'opera' )
		{
			header('Content-Disposition: attachment; filename="' . $filename . '"'); 
			header('Content-Description: File Transfert'); 
			header('Cache-Control: must-revalidate, post-check=0, pre-check=0'); 
			header('Pragma: public'); 
		}
		else
		{
			header('Content-Disposition: attachment; filename=' . $filename); 
			header('Pragma: no-cache'); 
		}
		
		if( $filesize )
		{
			header('Content-Length: ' . $filesize); 
		}
		
		echo $filedata; 
		
		exit;
	}
	
	/*
	 * Attach::quit()
	 * 
	 * Fermeture de connexion au serveur ftp
	 * 
	 * @return void
	 */
	function quit()
	{
		if( $this->use_ftp )
		{
			$quit = ( WA_PHP_VERSION >= 40200 ) ? 'ftp_close' : 'ftp_quit'; 
			
			@$quit($this->connect_id); 
		}
	}
}

}
?>